# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# STEPS - STochastic Engine for Pathway Simulation
# Copyright (C) 2007-2011 Okinawa Institute of Science and Technology, Japan.
# Copyright (C) 2003-2006 University of Antwerp, Belgium.
#
# See the file AUTHORS for details.
#
# This file is part of STEPS.
#
# STEPS is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# STEPS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import math
import numpy
import time 
from pylab import *

import steps.model as smod
import steps.geom as sgeom
import steps.rng as srng
import steps.solver as ssolv

from tol_funcs import *

########################################################################

KCST_f = 100/(6.022e23*1.0e-15) 			# The reaction constant, production
KCST_b = 10			# The reaction constant, degradation
VOL = 1.0e-18

DT = 0.1			# Sampling time-step
INT = 200000.1 		# Sim endtime

# Tolerance for the comparison:
# In tests with good code <1% fail with tolerance of 1.5%
tolerance = 1.5/100  

########################################################################

mdl  = smod.Model()

A = smod.Spec('A', mdl)

volsys = smod.Volsys('vsys',mdl)

# Production
R1 = smod.Reac('R1', volsys, lhs = [], rhs = [A], kcst = KCST_f)
R2 = smod.Reac('R2', volsys, lhs = [A], rhs = [], kcst = KCST_b)

geom = sgeom.Geom()

comp1 = sgeom.Comp('comp1', geom, VOL)
comp1.addVolsys('vsys')

rng = srng.create('mt19937', 1000)
rng.initialize(int(time.time()%4294967295))

sim = ssolv.Wmdirect(mdl, geom, rng)
sim.reset()

tpnts = numpy.arange(0.0, INT, DT)
ntpnts = tpnts.shape[0]

res = numpy.zeros([ntpnts])

sim.reset()
sim.setCompCount('comp1', 'A', 0)

for t in xrange(0, ntpnts):
    sim.run(tpnts[t])
    res[t] = sim.getCompCount('comp1', 'A')

def fact(x): return (1 if x==0 else x * fact(x-1))

# Do cumulative count, but not comparing them all. 
# Don't get over 50 (I hope)
steps_n_res = numpy.zeros(50)
for r in res: steps_n_res[r]+=1
for s in range(50): steps_n_res[s] = steps_n_res[s]/ntpnts

passed = True
max_err = 0.0

k1 = KCST_b
k2 = KCST_f*(6.022e23*1.0e-15)

# Compare 5 to 15
for m in range(5, 16):
    analy = (1.0/fact(m))*math.pow((k2/k1), m)*math.exp(-(k2/k1))
    if not tolerable(steps_n_res[m], analy, tolerance):
        passed = False
    if (abs(2*(steps_n_res[m]-analy)/(steps_n_res[m]+analy)) > max_err): max_err = abs(2*(steps_n_res[m]-analy)/(steps_n_res[m]+analy))
    #print abs(2*(steps_n_res[m]-analy)/(steps_n_res[m]+analy))         
#print "Max error: ", max_err*100, "%"




# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# STEPS - STochastic Engine for Pathway Simulation
# Copyright (C) 2007-2011 Okinawa Institute of Science and Technology, Japan.
# Copyright (C) 2003-2006 University of Antwerp, Belgium.
#
# See the file AUTHORS for details.
#
# This file is part of STEPS.
#
# STEPS is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# STEPS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import steps.model as smod
import steps.geom as sgeom
import steps.rng as srng
import steps.solver as ssolv

import math
import time 
import numpy
from pylab import *
import steps.utilities.meshio as meshio

from tol_funcs import *

########################################################################

NITER =	50			# The number of iterations
DT = 0.1			# Sampling time-step
INT = 0.3		# Sim endtime

DCSTA = 400*1e-12
DCSTB = DCSTA
RCST = 100000.0e6

#NA0 = 100000	# 1000000			# Initial number of A molecules
NA0 = 1000
NB0 = NA0		# Initial number of B molecules

SAMPLE = 1686

# <1% fail with a tolerance of 7.5%
tolerance = 7.5/100


# create the array of tet indices to be found at random
tetidxs = numpy.zeros(SAMPLE, dtype = 'int')
# further create the array of tet barycentre distance to centre
tetrads = numpy.zeros(SAMPLE)

########################################################################
rng = srng.create('mt19937', 512) 
rng.initialize(int(time.time()%4294967295)) # The max unsigned long


mdl  = smod.Model()

A = smod.Spec('A', mdl)
B = smod.Spec('B', mdl)


volsys = smod.Volsys('vsys',mdl)


R1 = smod.Reac('R1', volsys, lhs = [A,B], rhs = [])

R1.setKcst(RCST)

D_a = smod.Diff('D_a', volsys, A, dcst = DCSTA)
D_b = smod.Diff('D_b', volsys, B, dcst = DCSTB)


mesh = meshio.loadMesh('validation/meshes/brick_40_4_4_1686tets')[0]

VOLA = mesh.getMeshVolume()/2.0
VOLB = VOLA

ntets = mesh.countTets()

acomptets = []
bcomptets = []
max = mesh.getBoundMax()
min = mesh.getBoundMax()
midz = 0.0
compatris=set()
compbtris=set()
for t in range(ntets):
    barycz = mesh.getTetBarycenter(t)[0]
    tris = mesh.getTetTriNeighb(t)
    if barycz < midz: 
        acomptets.append(t)
        compatris.add(tris[0])
        compatris.add(tris[1])
        compatris.add(tris[2])
        compatris.add(tris[3])
    else: 
        bcomptets.append(t)
        compbtris.add(tris[0])
        compbtris.add(tris[1])
        compbtris.add(tris[2])
        compbtris.add(tris[3])

dbset = compatris.intersection(compbtris)
dbtris = list(dbset)

compa = sgeom.TmComp('compa', mesh, acomptets)
compb = sgeom.TmComp('compb', mesh, bcomptets)
compa.addVolsys('vsys')
compb.addVolsys('vsys')

diffb = sgeom.DiffBoundary('diffb', mesh, dbtris)


# Now fill the array holding the tet indices to sample at random
assert(SAMPLE <= ntets)

numfilled = 0
while (numfilled < SAMPLE):
    tetidxs[numfilled] = numfilled
    numfilled +=1

# Now find the distance of the centre of the tets to the Z lower face
for i in range(SAMPLE):
	baryc = mesh.getTetBarycenter(tetidxs[i])
	r = baryc[0]
	tetrads[i] = r*1.0e6

Atets = acomptets
Btets = bcomptets

rng = srng.create('mt19937', 16384)
rng.initialize(int(time.time()%4294967295))


sim = ssolv.Tetexact(mdl, mesh, rng)

sim.reset()

tpnts = numpy.arange(0.0, INT, DT)
ntpnts = tpnts.shape[0]

resA = numpy.zeros((NITER, ntpnts, SAMPLE))
resB = numpy.zeros((NITER, ntpnts, SAMPLE))


for i in xrange (0, NITER):
    sim.reset()
    
    sim.setDiffBoundaryDiffusionActive('diffb', 'A', True)
    sim.setDiffBoundaryDiffusionActive('diffb', 'B', True)

    sim.setCompCount('compa', 'A', NA0)
    sim.setCompCount('compb', 'B', NB0)

    for t in xrange(0, ntpnts):
        sim.run(tpnts[t])
        for k in range(SAMPLE):
            resA[i,t,k] = sim.getTetCount(tetidxs[k], 'A')
            resB[i,t,k] = sim.getTetCount(tetidxs[k], 'B')
	
	
itermeansA = numpy.mean(resA, axis=0)
itermeansB = numpy.mean(resB, axis=0)

import pylab


def getdetc(t, x):
    N = 1000		# The number to represent infinity in the exponential calculation
    L = 20e-6

    concA  = 0.0
    for n in xrange(N):
        concA+= ((1.0/(2*n +1))*math.exp((-(DCSTA/(20.0e-6))*math.pow((2*n +1), 2)*math.pow(math.pi, 2)*t)/(4*L))*math.sin(((2*n +1)*math.pi*x)/(2*L)))
    concA*=((4*NA0/math.pi)/(VOLA*6.022e26))*1.0e6	
    
    return concA


tpnt_compare = [1, 2]
passed = True
max_err = 0.0

for tidx in tpnt_compare:
    NBINS=10
    radmax = 0.0
    radmin = 10.0
    for r in tetrads:
        if (r > radmax): radmax = r
        if (r < radmin) : radmin = r
    
    rsec = (radmax-radmin)/NBINS
    binmins = numpy.zeros(NBINS+1)
    tetradsbinned = numpy.zeros(NBINS)
    r = radmin
    bin_vols = numpy.zeros(NBINS)
    
    for b in xrange(NBINS+1):
        binmins[b] = r
        if (b!=NBINS): tetradsbinned[b] = r +rsec/2.0
        r+=rsec
    
    bin_countsA = [None]*NBINS
    bin_countsB = [None]*NBINS
    for i in xrange(NBINS):
        bin_countsA[i] = []
        bin_countsB[i] = []
    filled = 0
    
    for i in xrange(itermeansA[tidx].size):
        irad = tetrads[i]
        
        for b in xrange(NBINS):
            if(irad>=binmins[b] and irad<binmins[b+1]):
                bin_countsA[b].append(itermeansA[tidx][i])
                bin_vols[b]+=sim.getTetVol(tetidxs[i])
                filled+=1.0
                break
    filled = 0
    for i in xrange(itermeansB[tidx].size):
        irad = tetrads[i]
        
        for b in xrange(NBINS):
            if(irad>=binmins[b] and irad<binmins[b+1]):
                bin_countsB[b].append(itermeansB[tidx][i])
                filled+=1.0
                break
    
    bin_concsA = numpy.zeros(NBINS)
    bin_concsB = numpy.zeros(NBINS)
    
    for c in range(NBINS): 
        for d in range(bin_countsA[c].__len__()):
            bin_concsA[c] += bin_countsA[c][d]
        for d in range(bin_countsB[c].__len__()):
            bin_concsB[c] += bin_countsB[c][d]        
        
        bin_concsA[c]/=(bin_vols[c])
        bin_concsA[c]*=(1.0e-3/6.022e23)*1.0e6    
        bin_concsB[c]/=(bin_vols[c])
        bin_concsB[c]*=(1.0e-3/6.022e23)*1.0e6 
    
    for i in range(NBINS):
        rad = abs(tetradsbinned[i])*1.0e-6
        
        if (tetradsbinned[i] < -5):
            # compare A
            det_conc = getdetc(tpnts[tidx], rad)
            steps_conc = bin_concsA[i]
            if not tolerable(det_conc, steps_conc, tolerance): passed = False
            if (abs(2*(det_conc-steps_conc)/(det_conc+steps_conc)) > max_err): max_err = abs(2*(det_conc-steps_conc)/(det_conc+steps_conc))
            #print "Error:",abs(2*(det_conc-steps_conc)/(det_conc+steps_conc))*100.0, "%"    
            #print ""    
        elif (tetradsbinned[i] > 5):
            # compare B
            det_conc = getdetc(tpnts[tidx], rad)
            steps_conc = bin_concsB[i]
            if not tolerable(det_conc, steps_conc, tolerance): passed = False
            if (abs(2*(det_conc-steps_conc)/(det_conc+steps_conc)) > max_err): max_err = abs(2*(det_conc-steps_conc)/(det_conc+steps_conc))
            #print "Error:",abs(2*(det_conc-steps_conc)/(det_conc+steps_conc))*100.0, "%"       
            #print "" 
#print "max error", max_err*100.0, "%"               




# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# STEPS - STochastic Engine for Pathway Simulation
# Copyright (C) 2007-2011 Okinawa Institute of Science and Technology, Japan.
# Copyright (C) 2003-2006 University of Antwerp, Belgium.
#
# See the file AUTHORS for details.
#
# This file is part of STEPS.
#
# STEPS is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# STEPS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

#  Last Changed Rev:  $Rev: 410 $
#  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
#  Last Changed By:   $Author: iain $

""" 
.. Note::

    This module is preliminary, means some of the functions are still under development.
    Code modification / debugging is wellcomed.
    Please email steps.dev@gmail.com if you would like to share you changes with others.

Visual Toolkit

"""

try:
    import wx
    import sys
    import  wx.lib.colourselect as  csel
    haveWXPython = True
except ImportError:
    haveWXPython = False


try:
    from wx import glcanvas
    haveGLCanvas = True
except ImportError:
    haveGLCanvas = False

try:
    from OpenGL.GL import *
    from OpenGL.GLU import *
    from OpenGL.GLUT import *
    haveOpenGL = True
except ImportError:
    haveOpenGL = False

try:
    import steps
    from steps import geom
    haveSTEPS = True
except ImportError:
    haveSTEPS = False

import time
import os   
import cPickle 

wildcard = "STEPS CheckPoint File (*.checkpoint)|*.checkpoint|"     \
           "All files (*.*)|*.*"
#----------------------------------------------------------------------
#
#                          Mesh Display
#
#----------------------------------------------------------------------

class MyCanvasBase(glcanvas.GLCanvas):
    def __init__(self, parent):
        glcanvas.GLCanvas.__init__(self, parent, -1)
        self.init = False
        # initial mouse position
        self.lastx = self.x = 0
        self.lasty = self.y = 0
        self.rotate = [20.0, 45.0, 0.0]
        self.centerx = 0.0
        self.centery = 0.0
        self.zooming = 1.0
        self.size = None
        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)
        self.Bind(wx.EVT_SIZE, self.OnSize)
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_LEFT_DOWN, self.OnMouseDown)
        self.Bind(wx.EVT_LEFT_UP, self.OnMouseUp)
        self.Bind(wx.EVT_MOTION, self.OnMouseMotion)
        self.Bind(wx.EVT_MOUSEWHEEL, self.OnMouseWheel)
        
    def OnEraseBackground(self, event):
        pass # Do nothing, to avoid flashing on MSW.


    def OnSize(self, event):
        size = self.size = self.GetClientSize()
        if self.GetContext():
            self.SetCurrent()
            glViewport(0, 0, size.width, size.height)
        event.Skip()


    def OnPaint(self, event):
        dc = wx.PaintDC(self)
        self.SetCurrent()
        if not self.init:
            self.InitGL()
            self.init = True
        self.OnDraw()
        
    def OnMouseDown(self, evt):
        self.freezexy = False
        self.CaptureMouse()
        self.x, self.y = self.lastx, self.lasty = evt.GetPosition()


    def OnMouseUp(self, evt):
        self.ReleaseMouse()
    
    def OnMouseMotion(self, evt):
        if evt.Dragging() and evt.LeftIsDown():
            self.lastx, self.lasty = self.x, self.y
            self.x, self.y = evt.GetPosition()
            if self.x > self.lastx:
                self.centerx += 0.5 * self.zooming
            elif self.x < self.lastx:
                self.centerx -= 0.5 * self.zooming
            if self.y > self.lasty:
                self.centery -= 0.5 * self.zooming
            elif self.y < self.lasty:
                self.centery += 0.5 * self.zooming
            self.Refresh(False)
                
    def OnMouseWheel(self, evt):
        if evt.GetWheelRotation() > 0:
            self.zooming = self.zooming * 0.9
        else:
            self.zooming = self.zooming * 1.1
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        glOrtho(self.boundmin * self.zooming, self.boundmax * self.zooming, self.boundmin * self.zooming, self.boundmax * self.zooming, self.boundmin, self.boundmax)
        glMatrixMode (GL_MODELVIEW)
        self.OnDraw()

    def ResetDisp(self):
        self.rotate = [0.0, 0.0, 0.0]
        self.centerx = 0.0
        self.centery = 0.0
        self.zooming = 1.0
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        glOrtho(self.boundmin * self.zooming, self.boundmax * self.zooming, self.boundmin * self.zooming, self.boundmax * self.zooming, self.boundmin, self.boundmax)
        glMatrixMode (GL_MODELVIEW)
        self.OnDraw()
    
class MeshCanvas(MyCanvasBase):
    def __init__(self, parent, mesh, solver, scale = 1e-6):
        self.parent = parent
        self.solver = solver
        self.mesh = mesh
        self.scale = scale
        self.bgcolor = [255, 255, 255]
        self.meshcolor = [0, 0, 0]
        self.meshalpha = 255
        self.specs_data = {}
        self.pointsize = 4.0
        self.__loadMesh__()
        self.updateSpecies(parent.spec_color_mapping)
        MyCanvasBase.__init__(self, parent)

    def __loadMesh__(self):
        self.ntets = self.mesh.ntets
        self.nverts = self.mesh.nverts
        self.ntris = self.mesh.ntris
        self.verts = []
        self.tris = []
        self.patchtris = []
        self.patches = {}
        self.tribounds = self.mesh.getSurfTris()
        boundmin = self.mesh.getBoundMin()
        boundmax = self.mesh.getBoundMax()
        bmin = min(boundmin)
        bmax = max(boundmax)
        self.boundmin = bmin * 1.2 / self.scale
        self.boundmax = bmax * 1.2 / self.scale

        for n in range(self.nverts):
            unscaled = self.mesh.getVertex(n)
            self.verts.append([unscaled[0]/self.scale,unscaled[1]/self.scale,unscaled[2]/self.scale])
        for n in range(self.ntris):
            self.tris.append(self.mesh.getTri(n))
            patch = self.mesh.getTriPatch(n)
            if patch != None:
                self.patchtris.append(n)
                if patch.getID() in self.patches:
                    self.patches[patch.getID()].append(n)
                else:
                    self.patches[patch.getID()] = [n]

    def InitGL(self):
        # set viewing projection
        glMatrixMode(GL_PROJECTION)
        glOrtho(self.boundmin, self.boundmax, self.boundmin, self.boundmax, self.boundmin, self.boundmax)

        # position viewer
        glMatrixMode(GL_MODELVIEW)
        #glTranslatef(0.0, 0.0, -2.0)

        glEnable(GL_DEPTH_TEST)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
        glEnable( GL_BLEND )
        #glEnable(GL_LIGHTING)
        #glEnable(GL_LIGHT0)


    def OnDraw(self):
        # clear color and depth buffers
        glClearColor(self.bgcolor[0]/255.0, self.bgcolor[1]/255.0, self.bgcolor[2]/255.0, 1.0)
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()      
        
        glTranslatef(self.centerx, self.centery, 0)
        
        glRotatef(self.rotate[0], 1.0, 0.0, 0.0)
        glRotatef(self.rotate[1], 0.0, 1.0, 0.0) 
        glRotatef(self.rotate[2], 0.0, 0.0, 1.0) 
        
        # draw species
        glEnableClientState(GL_VERTEX_ARRAY)
        glPointSize(self.pointsize)
        for key in self.specs_data.keys():
            glColor3ubv(self.parent.spec_color_mapping[key][1])
            glVertexPointer(3, GL_DOUBLE, 0, self.specs_data[key])
            nverts = len(self.specs_data[key]) / 3
            glDrawElements(GL_POINTS, nverts, GL_UNSIGNED_BYTE, range(nverts)) 
               
        glDisableClientState(GL_VERTEX_ARRAY)
        
        glPointSize(1)        
        # draw mesh
        colorvec = [self.meshcolor[0],self.meshcolor[1],self.meshcolor[2],self.meshalpha]
        glColor4ubv(colorvec)
        glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)
        glBegin(GL_TRIANGLES)
        for n in range(len(self.tribounds)):
            tri = self.tris[self.tribounds[n]]
            for v in range(3):
                vert = tri[v]
                glVertex3f( self.verts[vert][0], self.verts[vert][1], self.verts[vert][2])
        for patch in self.patches:
            for t in self.patches[patch]:
                tri = self.tris[t]
                for v in range(3):
                    vert = tri[v]
                    glVertex3f( self.verts[vert][0], self.verts[vert][1], self.verts[vert][2])
        glEnd()

        glFlush()
        self.SwapBuffers()
    
    def updateSpecies(self, spec_color_mapping):
        self.specs_data.clear()
        for key in spec_color_mapping.keys():
            if spec_color_mapping[key][0]:
                self.specs_data[key] = []
                
        for t in range(self.ntets):
            for key in self.specs_data.keys():
                if(self.solver.getTetSpecDefined(t, key)):
                    if(self.solver.getTetCount(t, key) > 0):
                        center = self.mesh.getTetBarycenter(t)
                        scaled_center = [center[0]/self.scale, center[1]/self.scale, center[2]/self.scale]
                        self.specs_data[key].extend(scaled_center)
        
        for t in self.patchtris:
            for key in self.specs_data.keys():
                if(self.solver.getTriSpecDefined(t, key)):
                    if(self.solver.getTriCount(t, key) > 0):
                        center = self.mesh.getTriBarycenter(t)
                        scaled_center = [center[0]/self.scale, center[1]/self.scale, center[2]/self.scale]
                        self.specs_data[key].extend(scaled_center)
#----------------------------------------------------------------------
#
#                           GUI Simulation Frontend
#
#----------------------------------------------------------------------

class SimCtrlPanel(wx.Panel):
    def __init__(self, parent, model, mesh, solver):
        wx.Panel.__init__(self, parent)
        self.parent = parent
        self.model = model
        self.mesh = mesh
        self.solver = solver
        self.spec_list = []
        
        # layout
        box = wx.BoxSizer(wx.VERTICAL)
        
        box.AddSpacer(10)
        box.Add(wx.StaticText(self, -1, "Simulation Time (Sec)"), 0, wx.ALIGN_CENTER, border = 10)
        self.simtime_txt = wx.StaticText(self, -1, "%e" % (self.solver.getTime()))
        box.AddSpacer(10)
        box.Add(self.simtime_txt, 0, wx.ALIGN_CENTER, border = 10)
                      
        static1 = wx.StaticText(self, -1, "Stop at (Sec)")
        self.stoptime_text = wx.TextCtrl(self,-1, "1e-3")
        box.AddSpacer(10)
        box.Add(static1, 0, wx.ALIGN_CENTER, border = 10)
        box.AddSpacer(10)
        box.Add(self.stoptime_text, 0, wx.ALIGN_CENTER, border = 10)
        
        
        static2 = wx.StaticText(self, -1, "Advance Interval (Sec)")
        self.advance_text = wx.TextCtrl(self,-1, "1e-5")
        
        box.AddSpacer(10)
        box.Add(static2, 0, wx.ALIGN_CENTER, border = 10)
        box.AddSpacer(10)
        box.Add(self.advance_text, 0, wx.ALIGN_CENTER, border = 10)
        

        self.runbtn = wx.Button(self, -1, "Run")
        box.AddSpacer(10)
        box.Add(self.runbtn, 0, wx.ALIGN_CENTER, border = 10)
        
        #self.replaybtn = wx.Button(self, -1, "Replay")
        #box.AddSpacer(10)
        #box.Add(self.replaybtn, 0, wx.ALIGN_CENTER, border = 10)

        box.AddSpacer(10)
        btn_box = wx.BoxSizer(wx.HORIZONTAL)
        self.savebtn = wx.Button(self, -1, "Save")
        btn_box.AddSpacer(5)
        btn_box.Add(self.savebtn, 0, wx.ALIGN_CENTER, border = 10)
        
        self.loadbtn = wx.Button(self, -1, "Load")
        btn_box.AddSpacer(5)
        btn_box.Add(self.loadbtn, 0, wx.ALIGN_CENTER, border = 10)
        
        box.Add(btn_box, 0, wx.ALIGN_CENTER, border = 5)
        
        box.AddSpacer(20)
        box.Add(wx.StaticText(self, -1, "Species"), 0, wx.ALIGN_CENTER, border = 10)
        box.AddSpacer(10)
        self.spec_listbox = wx.ListBox(self, -1, style = wx.LB_SINGLE)
        box.Add(self.spec_listbox, 1, wx.ALIGN_CENTER, border = 10)
        
        
        color_box = wx.BoxSizer(wx.HORIZONTAL)
        color_box.AddSpacer(10)
        self.specdisp_check = wx.CheckBox(self, -1, "Display")
        color_box.Add(self.specdisp_check, 0, wx.ALIGN_CENTER, border = 10)
        self.speccolor_select = csel.ColourSelect(self,-1, colour = wx.RED)
        color_box.AddSpacer(10)
        color_box.Add(self.speccolor_select, 0, wx.ALIGN_CENTER, border = 10)
        self.specsize_spin = wx.SpinCtrl(self, -1, "Size")
        self.specsize_spin.SetRange(1,100)
        self.specsize_spin.SetValue(4)
        
        
        box.AddSpacer(10)
        box.Add(color_box, 0, wx.ALIGN_CENTER, border = 10)
        
        size_box = wx.BoxSizer(wx.HORIZONTAL)
        size_box.AddSpacer(10)
        size_box.Add(wx.StaticText(self, -1, "Spec Size"), 0, wx.ALIGN_CENTER, border = 10)
        size_box.AddSpacer(10)
        size_box.Add(self.specsize_spin, 0, wx.ALIGN_CENTER, border = 10)
        
        box.AddSpacer(5)
        box.Add(size_box, 0, wx.ALIGN_CENTER, border = 10)

        mesh_box = wx.BoxSizer(wx.HORIZONTAL)
        mesh_box.Add(wx.StaticText(self, -1, "Mesh Color"), 0, wx.ALIGN_CENTER, border = 10)
        self.meshcolor_select = csel.ColourSelect(self,-1, colour = wx.BLACK)
        mesh_box.AddSpacer(20)
        mesh_box.Add(self.meshcolor_select, 0, wx.ALIGN_CENTER, border = 10)
        
        box.AddSpacer(20)
        box.Add(mesh_box, 0, wx.ALIGN_CENTER, border = 10)
        
        box.AddSpacer(5)
        box.Add(wx.StaticText(self, -1, "Transparency"), 0, wx.ALIGN_CENTER, border = 10)
        
        self.alpha_slider = wx.Slider(self, -1, 255, 0, 255, style = wx.SL_HORIZONTAL | wx.SL_AUTOTICKS | wx.SL_LABELS)
        box.Add(self.alpha_slider, 0, wx.ALIGN_CENTER, border = 10)
        
        background_box = wx.BoxSizer(wx.HORIZONTAL)
        background_box.Add(wx.StaticText(self, -1, "BG Color"), 0, wx.ALIGN_CENTER, border = 10)
        self.backgroundcolor_select = csel.ColourSelect(self,-1, colour = wx.WHITE)
        background_box.AddSpacer(10)
        background_box.Add(self.backgroundcolor_select, 0, wx.ALIGN_CENTER, border = 10)
        
        box.AddSpacer(10)
        box.Add(background_box, 0, wx.ALIGN_CENTER, border = 10)
        
        box.AddSpacer(10)
        box.Add(wx.StaticText(self, -1, "Rotation"), 0, wx.ALIGN_CENTER, border = 10)
        self.rotatebox = wx.RadioBox(self, -1, "", wx.DefaultPosition, wx.DefaultSize,["X","Y", "Z"], 1, wx.RA_SPECIFY_ROWS)
        box.Add(self.rotatebox, 0, wx.ALIGN_CENTER, border = 10)
        self.rotate_slider = wx.Slider(self, -1, 20, -180, 180, style = wx.SL_HORIZONTAL | wx.SL_AUTOTICKS | wx.SL_LABELS)
        box.Add(self.rotate_slider, 0, wx.ALIGN_CENTER, border = 10)
        self.resetbtn = wx.Button(self, -1, "Reset")
        box.Add(self.resetbtn, 0, wx.ALIGN_CENTER, border = 10)
        
        box.AddSpacer(20)             
        self.SetSizerAndFit(box)
        
        # event binding
        self.meshcolor_select.Bind(csel.EVT_COLOURSELECT, self.OnMeshColorChange, id=self.meshcolor_select.GetId())
        self.alpha_slider.Bind(wx.EVT_SCROLL, self.OnMeshAlphaChange, id = self.alpha_slider.GetId())
        self.backgroundcolor_select.Bind(csel.EVT_COLOURSELECT, self.OnBGColorChange, id=self.backgroundcolor_select.GetId())
        self.spec_listbox.Bind(wx.EVT_LISTBOX, self.OnSpecSelectChange, id = self.spec_listbox.GetId())
        self.specdisp_check.Bind(wx.EVT_CHECKBOX, self.OnSpecDispChange, id = self.specdisp_check.GetId())
        self.speccolor_select.Bind(csel.EVT_COLOURSELECT, self.OnSpecColorChange, id=self.speccolor_select.GetId())
        self.specsize_spin.Bind(wx.EVT_SPINCTRL, self.OnSpecSizeChange, id = self.specsize_spin.GetId())
        self.runbtn.Bind(wx.EVT_BUTTON, self.OnRun, id = self.runbtn.GetId())
        self.rotatebox.Bind(wx.EVT_RADIOBOX, self.OnCordChange, id = self.rotatebox.GetId())
        self.rotate_slider.Bind(wx.EVT_SCROLL, self.OnRotateDisp, id = self.rotate_slider.GetId()) 
        self.resetbtn.Bind(wx.EVT_BUTTON, self.OnResetView, id = self.resetbtn.GetId())
        self.savebtn.Bind(wx.EVT_BUTTON, self.OnSave, id = self.savebtn.GetId())
        self.loadbtn.Bind(wx.EVT_BUTTON, self.OnLoad, id = self.loadbtn.GetId())       
        #self.replaybtn.Bind(wx.EVT_BUTTON, self.OnReplay, id = self.replaybtn.GetId()) 
        
    def refreshSpecList(self, list):
        self.spec_list = list
        self.spec_listbox.Set(list)
    
    def OnMeshColorChange(self, event):
        self.parent.disp_panel.meshcolor = list(event.GetValue())
        self.parent.disp_panel.OnDraw()
        
    def OnMeshAlphaChange(self, event):
        self.parent.disp_panel.meshalpha = self.alpha_slider.GetValue()
        self.parent.disp_panel.OnDraw()
    
    def OnBGColorChange(self, event):
        self.parent.disp_panel.bgcolor = list(event.GetValue())
        self.parent.disp_panel.OnDraw()
        
    def OnSpecSelectChange(self, event):
        specstatus = self.parent.spec_color_mapping[event.GetString()]
        self.specdisp_check.SetValue(specstatus[0])
        self.speccolor_select.SetColour(specstatus[1])
    
    def OnSpecDispChange(self, event):
        specid = self.spec_listbox.GetSelections()
        if len(specid) == 1:
            specid = specid[0]
        else:
            return
        if specid >= 0 and specid < len(self.spec_list):
            self.parent.spec_color_mapping[self.spec_list[specid]][0] = event.IsChecked()
            self.parent.updateSpecs()
            self.parent.disp_panel.OnDraw()
        
    def OnSpecColorChange(self, event):
        specid = self.spec_listbox.GetSelections()
        if len(specid) == 1:
            specid = specid[0]
        else:
            return
        if specid >= 0 and specid < len(self.spec_list):
            self.parent.spec_color_mapping[self.spec_list[specid]][1] = list(event.GetValue())
            self.parent.updateSpecs()
            self.parent.disp_panel.OnDraw()
        
    def OnSpecSizeChange(self,event):
        self.parent.disp_panel.pointsize = self.specsize_spin.GetValue()
        self.parent.disp_panel.OnDraw()
    
    def OnCordChange(self, event):
        cord = event.GetInt()
        self.rotate_slider.SetValue(self.parent.disp_panel.rotate[cord])
    
    def OnRotateDisp(self, event):
        cord = self.rotatebox.GetSelection()
        self.parent.disp_panel.rotate[cord] = self.rotate_slider.GetValue()
        self.parent.disp_panel.OnDraw()
    
    def OnResetView(self, event):
        self.parent.disp_panel.ResetDisp()
        self.rotate_slider.SetValue(0)
    
    def OnRun(self, event):
        while self.solver.getTime() < float(self.stoptime_text.GetValue()):
            self.solver.advance(float(self.advance_text.GetValue()))
            self.simtime_txt.SetLabel("%e" % (self.solver.getTime()))
            self.parent.updateSpecs()
            self.parent.disp_panel.OnDraw()
            self.Update()
            
    def OnSave(self,event):
        dlg = wx.FileDialog(
            self, message="Save file as ...", defaultDir=os.getcwd(), 
            defaultFile="Untitled.checkpoint", wildcard=wildcard, style=wx.SAVE
            )

        dlg.SetFilterIndex(2)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            self.solver.checkpoint(path)
        dlg.Destroy()
        
    def OnLoad(self, event):
        dlg = wx.FileDialog(
            self, message="Choose a file",
            defaultDir=os.getcwd(), 
            defaultFile="",
            wildcard=wildcard,
            style=wx.OPEN | wx.CHANGE_DIR
            )

        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            self.solver.restore(path)
            self.simtime_txt.SetLabel("%e" % (self.solver.getTime()))
            self.parent.updateSpecs()
            self.parent.disp_panel.OnDraw()
        dlg.Destroy()
 
    #def OnReplay(self, event):
    #    dlg = wx.FileDialog(
    #        self, message="Choose a file",
    #        defaultDir=os.getcwd(), 
    #        defaultFile="",
    #        style=wx.OPEN | wx.CHANGE_DIR
    #        )
#
 #       if dlg.ShowModal() == wx.ID_OK:
  #          path = dlg.GetPath()
    #        self.solver.restore(path)
   #         self.simtime_txt.SetLabel("%e" % (self.solver.getTime()))
    #        self.parent.updateSpecs()
    #        self.parent.disp_panel.OnDraw()
     #   dlg.Destroy()
        
class SimPanel(wx.SplitterWindow):
    def __init__(self, parent, model, mesh, solver, scale = 1e-6, color_map = [[255,0,0], [0,0,255]]):
        wx.SplitterWindow.__init__(self, parent, -1)
        
        #get species list and initialise color mapping
        species = model.getAllSpecs()
        self.spec_color_mapping = {}
        
        for s in range(len(species)):
            if s < len(color_map):
                self.spec_color_mapping[species[s].id] = [True, color_map[s]]
            else:
                self.spec_color_mapping[species[s].id] = [True, [0,255,0]]
                
        self.SetAutoLayout(True)
        
        self.ctrl_panel = SimCtrlPanel(self, model, mesh, solver)
        self.ctrl_panel.refreshSpecList(self.spec_color_mapping.keys())
        self.disp_panel = MeshCanvas(self, mesh, solver, scale)
        
        self.SetMinimumPaneSize(100)
        self.SplitVertically(self.disp_panel, self.ctrl_panel, -100)
        
    def updateSpecs(self):
        self.disp_panel.updateSpecies(self.spec_color_mapping)
    

def GUISim(model, mesh, solver, scale = 1e-6, color_map = [[255,0,0], [0,0,255]]):
    """
    Create a graphical frontend for a mesh based simulation.
    
    .. Note::
    
        The graphical frontend should be used for mesh based simulation only.
    
    Arguements:
        * steps.model.Model model
        * steps.geom.Tetmesh mesh
        * steps.solver.Tetexact solver
        
    Example:
        see examples/diffusion
    """
    
    if not haveSTEPS:
        print("Cannot find STEPS! Please install it first.")
        return
    
    if not haveWXPython:
        print("Cannot find WxPython! Please install it first.")
        return
        
    if not haveOpenGL:
        print("Cannot find PyOpenGL! Please install it first.")
        return
        
        
    app = wx.App(False)
    frame = wx.Frame(None, wx.ID_ANY, 'STEPS Visual Frontend for Mesh Based Simulation http://steps.sourceforge.net', size = (1200,900), style=wx.DEFAULT_FRAME_STYLE)
    sim_panel = SimPanel(frame, model, mesh, solver, scale, color_map)
    frame.Centre()
    frame.Show()
    app.MainLoop()
    


# -*- coding: utf-8 -*-

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# STEPS - STochastic Engine for Pathway Simulation
# Copyright (C) 2007-2011 Okinawa Institute of Science and Technology, Japan.
# Copyright (C) 2003-2006 University of Antwerp, Belgium.
#
# See the file AUTHORS for details.
#
# This file is part of STEPS.
#
# STEPS is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# STEPS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

#  Last Changed Rev:  $Rev: 410 $
#  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
#  Last Changed By:   $Author: iain $

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# This file is the user-interface file for all geom objects in STEPS.
# All objects are directly derived from the corresponding swig objects.
# Geom and Tetmesh(derived from Geom) container objects are owned by Python
# All other objects are owned by c++ and container is responsible for 
# all the cleaning-up of these objects (see cpp/geom/geom.cpp class 
# destructor).
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #


from . import steps_swig
import _steps_swig

### Now defunct mesh saving/loading tool ###
# from steps_swig import loadASCII, saveASCII


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

class Geom(steps_swig.Geom):
    """
    Top-level geometry container to which a number of compartment objects 
    and patches objects may be grouped. 
    
    A steps.geom.Geom object is parent to the following objects:

    * steps.geom.Comp
    * steps.geom.Patch
    """
    def __init__(self, *args): 
        """
        Construction::
        
            g = steps.geom.Geom()
            
        Create a geometry container object.
            
        Arguments: 
            None
        """
        this = _steps_swig.new_Geom(*args)
        try: self.this.append(this)
        except: self.this = this
        # set Geom object to do all the cleaning-up
        self.thisown = True


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 


class Patch(steps_swig.Patch):
    """
    Base class for patch objects. A patch is a piece of 2D surface surrounding 
    (part of) a 3D compartment, which may be connected to another compartment. 
    It provides basic functionality and descriptive data that is shared by the derived 
    class steps.geom.TmPatch (that is used to describe a surface comprised of 
    triangles in a tetrahedral mesh):

    * Getting and setting a valid patch identifier string, and handling 
      the interaction with the container object.
    * Getting (and at least in this base class also setting) the total area of the patch.
    * Adding surface systems to the patch.
    * References to inside/outside compartments to which the patch is adjoined.

    This base class can be used directly with well-mixed solvers. 
    """
    def __init__(self, *args, **kwargs): 
        """
        Construction::
        
            patch = steps.geom.Patch(id, container, icomp, ocomp = None, area = 0.0)
            
        Construct a patch object with identifier string id, assign container 
        as the parent geometry container and assign icomp as the "inner" 
        compartment. Assign also ocomp as the "outer" compartment (if required) 
        and optionally set the area to area (in m^2). 
                    
        Arguments: 
            * string id
            * steps.geom.Geom container
            * steps.geom.Comp icomp
            * steps.geom.Comp ocomp (default = None)
            * float area (default = 0.0)
            
        .. note: "Inner" compartment and "outer" compartment are purely defined 
           by their order to the class constructor.
        
        """
        this = _steps_swig.new_Patch(*args, **kwargs)
        try: self.this.append(this)
        except: self.this = this
        self.thisown = False
        self.__swig_setmethods__["id"] = _steps_swig.Patch_setID
        self.__swig_getmethods__["id"] = _steps_swig.Patch_getID
        self.__swig_getmethods__["container"] = _steps_swig.Patch_getContainer
        self.__swig_getmethods__["surfsys"] = _steps_swig.Patch_getSurfsys
        self.__swig_setmethods__["area"] = _steps_swig.Patch_setArea
        self.__swig_getmethods__["area"] = _steps_swig.Patch_getArea
        self.__swig_getmethods__["icomp"] = _steps_swig.Patch_getIComp
        self.__swig_getmethods__["ocomp"] = _steps_swig.Patch_getOComp
        
    id = steps_swig._swig_property(_steps_swig.Patch_getID, _steps_swig.Patch_setID)
    """Identifier string of the patch."""
    container = steps_swig._swig_property(_steps_swig.Patch_getContainer)
    """Reference to parent steps.geom.Geom container."""
    surfsys = steps_swig._swig_property(_steps_swig.Patch_getSurfsys)
    """Reference to assocated surface system."""
    area = steps_swig._swig_property(_steps_swig.Patch_getArea, _steps_swig.Patch_setArea)
    """Area of the patch."""
    icomp = steps_swig._swig_property(_steps_swig.Patch_getIComp)
    """Reference to the inner compartment."""
    ocomp = steps_swig._swig_property(_steps_swig.Patch_getOComp)
    """reference to the outer compartment."""


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 


class Comp(steps_swig.Comp):
    """
    Base class for compartment objects. It provides basic functionality and data 
    that is shared by the derived class steps.geom.TmComp (that is used to 
    describe a compartment in a tetrahedral mesh):
    
    * Getting and setting a valid compartment identifier string, and handling 
      the interaction with the container object. 
    * Getting (and at least in this base class also setting) the total volume 
      of the compartment. 
    * Adding volume systems to the compartment. 
    * References to steps.geom.Patch objects to which the compartment is adjoined.
    
    This base class can be used directly with well-mixed solvers.
    
    **Relationship between Compartments and Patches**
    
    It is necessary to explain the inner/outer relationship between compartments 
    and patches. When a patch object is created (a surface of a compartment, which 
    may be shared with another compartment). it is necessary to arbitrarily label the 
    compartment(s) "inner" and "outer" (if a patch is connected to only one compartment
    then the compartment must be labelled "inner" by convention). This is necessary 
    in order to fully describe the surface reaction rules. Accordingly, compartments 
    also store a list of connections, "inner" patches and "outer" patches. So if a 
    patch1 is created with comp1 as it's "inner" compartment, comp1 knows patch 1 as 
    an "outer" patch. The labelling is purely defined when creating the Patch objects, 
    bearing in mind the stoichiometry defined in the surface reaction objects. This may 
    seem a little confusing at first, but will become clearer when experience is gained 
    with these objects.
     
    .. seealso:: :ref:`ip3`
     
    """     
    def __init__(self, *args, **kwargs): 
        """
        Construction::
        
            comp = steps.geom.Comp(id, container, vol = 0.0)
            
        Construct a compartment object with identifier string id and assign 
        container as the parent geometry container. Optionally set volume 
        to vol (in m^3).
            
        Arguments: 
            * string id
            * steps.geom.Geom container
            * float vol (default = 0.0)
        """
        this = _steps_swig.new_Comp(*args, **kwargs)
        try: self.this.append(this)
        except: self.this = this
        self.thisown = False
        self.__swig_setmethods__["id"] = _steps_swig.Comp_setID
        self.__swig_getmethods__["id"] = _steps_swig.Comp_getID
        self.__swig_getmethods__["container"] = _steps_swig.Comp_getContainer
        self.__swig_getmethods__["volsys"] = _steps_swig.Comp_getVolsys
        self.__swig_setmethods__["vol"] = _steps_swig.Comp_setVol
        self.__swig_getmethods__["vol"] = _steps_swig.Comp_getVol
        self.__swig_getmethods__["ipatches"] = _steps_swig.Comp_getIPatches
        self.__swig_getmethods__["opatches"] = _steps_swig.Comp_getOPatches
    
    id = steps_swig._swig_property(_steps_swig.Comp_getID, _steps_swig.Comp_setID)
    """Identifier string of the compartment."""
    container = steps_swig._swig_property(_steps_swig.Comp_getContainer)
    """Reference to parent steps.geom.Geom container."""
    volsys = steps_swig._swig_property(_steps_swig.Comp_getVolsys)
    """Reference to assocated volume system."""
    vol = steps_swig._swig_property(_steps_swig.Comp_getVol, _steps_swig.Comp_setVol)
    """Volume of the compartment."""
    ipatches = steps_swig._swig_property(_steps_swig.Comp_getIPatches)
    """List of reference to inner patches."""
    opatches = steps_swig._swig_property(_steps_swig.Comp_getOPatches)
    """List of reference to outer patches."""


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 


class Tetmesh(steps_swig.Tetmesh):
    """
    Main container class for static tetrahedral meshes. This class stores the 
    vertices points, 3D tetrahedral and 2D triangular elements that comprise 
    the mesh. The indices of the elements will be stored as unsigned integers 
    (a positive integer or zero) beginning at zero and incremented by 1. For 
    example, if there are ntets number of tetrahedrons in the mesh, the indices 
    of the tetrahedrons will be [0,1,2,..., (ntets-1)]. In addition, this class 
    computes and contains some auxiliary data from the mesh:

    * Rectangular, axis-aligned bounding box.
    * Overall volume.
    * The total number of tetrahedrons in the mesh.
    * The total number of triangles in the mesh.
    * The total number of vertices in the mesh.

    Auxiliary data is also stored for each tetrahedron:
    
    * Volume of the tetrahedron.
    * Indices of the 4 neighbouring tetrahedrons. If there is no neighbour 
      (i.e. if the tetrahedron lies on the border), this index will be -1. 
      The sequence of neighbours is determined by the following common boundary 
      triangles: (0,1,2), (0,1,3), (0,2,3), (1,2,3).
    * Indices of the 4 neighbouring boundary triangles. The sequence of neighbours 
      is also determined by (0,1,2), (0,1,3), (0,2,3), (1,2,3).
    * Compartment (steps.geom.TmComp object) that the tetrahedron belongs to. 
      Stores zero pointer if the tetrahedron has not been assigned to a compartment.

    And for each triangle:
    
    * Area of the triangle.
    * Normal vector of the triangle, normalized to length 1.0.
    * Indices of the 2 neighbouring tetrahedrons. If one tetrahedron does not exist 
      (because the triangle lies on the outer boundary), this index will be -1.
    * Patch (steps.geom.TmPatch object) that a triangle belongs to. Stores zero pointer 
      if triangle has not been assigned to a patch.
    """
    def __init__(self, *args): 
        """
        Construction1::
        
            mesh = steps.geom.Tetmesh(verts, tets, tris)
            
        Construct a Tetmesh container by the “first” method: Supply a list of all 
        vertices verts (by Cartesian coordinates), supply a list of all tetrahedrons 
        tets (by indices of the 4 vertices) and supply a full or partial list of 
        triangles tris (by indices of the 3 vertices). Indexing in STEPS begins at 
        zero, so the first 3 coordinates in verts will describe the zeroth vertex, t
        he next 3 coordinates will describe the 1st vertex and so on. Labelling of 
        the vertices in tets and tris should follow this indexing. Lists must be 
        one-dimensional. Length of verts = nverts*3 where nverts is the total 
        number of vertices; length of tets = ntets*4 where ntets is the total 
        number of tetrahedrons; maximum length of tris ntris*3 where ntris is 
        the total number of triangles. For example, if we have just three tetrahedrons; 
        tet0=[0,1,2,3], tet1=[0,1,3,4] and tet2=[1,3,4,5] then the required 
        one-dimensional list tets=[0,1,2,3,0,1,3,4,1,3,4,5]. 
            
        Arguments: 
            * list<float> verts
            * list<uint> tets
            * list<unit> tris
            
        Construction2::
            mesh = steps.geom.Tetmesh(nverts, ntets, ntris)
            
        Construct a Tetmesh container by the “second” method: Supply only the 
        number of vertices nverts, the number of tetrahedrons ntets and the number 
        of triangles ntris to the initializer, then use set methods to supply the 
        vertex, tetrahedron and triangle information one by one. It is up to the 
        user to make sure all information is supplied and then call setup() explicitly. 
        It is highly recommended to use the first constructor wherever possible due to t
        he scope for user error when using this method.
        
        Arguments: 
            * uint nverts
            * uint ntets
            * uint ntris
        """        
        this = _steps_swig.new_Tetmesh(*args)
        try: self.this.append(this)
        except: self.this = this
        # set Tetmesh object to do all the cleaning up
        self.thisown = True
        self.__swig_getmethods__["nverts"] = _steps_swig.Tetmesh_countVertices
        self.__swig_getmethods__["ntris"] = _steps_swig.Tetmesh_countTris
        self.__swig_getmethods__["ntets"] = _steps_swig.Tetmesh_countTets
    
    nverts = steps_swig._swig_property(_steps_swig.Tetmesh_countVertices)
    """Number of vertices in the mesh."""
    ntris = steps_swig._swig_property(_steps_swig.Tetmesh_countTris)
    """Number of triangles in the mesh."""
    ntets = steps_swig._swig_property(_steps_swig.Tetmesh_countTets)
    """Number of tetrahedrons in the mesh."""


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

"""
////////////////////////////////////////////////////////////////////////////////
//////// OBJECT REMOVED BECAUSE OF MEMORY ISSUES. SEE TODO NOTE IN C++ /////////
///////////////////////// CONSTRUCTOR FOR DETAILS //////////////////////////////
////////////////////////////////////////////////////////////////////////////////
"""

# class Tet(steps_swig.Tet):
#    def __init__(self, *args): 
#        """__init__(self, Tetmesh mesh, unsigned int tidx) -> Tet"""
#        this = _steps_swig.new_Tet(*args)
#        try: self.this.append(this)
#        except: self.this = this
#        # this object owned by python
#        self.thisown = True


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

"""
////////////////////////////////////////////////////////////////////////////////
//////// OBJECT REMOVED BECAUSE OF MEMORY ISSUES. SEE TODO NOTE IN C++ /////////
///////////////////////// CONSTRUCTOR FOR DETAILS //////////////////////////////
////////////////////////////////////////////////////////////////////////////////
"""

# class Tri(steps_swig.Tri):
#
#    def __init__(self, *args): 
#        """__init__(self, Tetmesh mesh, unsigned int tidx) -> Tri"""
#        this = _steps_swig.new_Tri(*args)
#        try: self.this.append(this)
#        except: self.this = this
#        # this object is owned by python
#        self.thisown = True


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 


class TmComp(steps_swig.TmComp):
    """
    Derived class from base steps.geom.Comp class. It provides the same 
    functionality as the steps.geom.Comp class extended for annotation of 
    a group of tetrahedrons in a Tetmesh. The volume is the total volume of 
    the encapsulated tetrahedrons.
    """
    def __init__(self, *args, **kwargs): 
        """
        Construction::
        
            tmcomp = steps.geom.Comp(id, container, tets)
            
        Construct a TmComp object with identifier string id and assign container 
        as the parent Tetmesh container. Set the group of tetrahedrons that describe 
        this compartment with tets.
            
        Arguments: 
            * string id
            * steps.geom.Tetmesh container
            * list<uint> tets
        """
        this = _steps_swig.new_TmComp(*args, **kwargs)
        try: self.this.append(this)
        except: self.this = this
        self.thisown = False
        self.__swig_getmethods__["tets"] = _steps_swig.TmComp_getAllTetIndices
    
    tets = steps_swig._swig_property(_steps_swig.TmComp_getAllTetIndices)
    """List of indices of tetrahedrons associated to the compartment."""


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 


class TmPatch(steps_swig.TmPatch):
    """
    Derived class from base steps.geom.Patch class. It provides the same 
    functionality as the steps.geom.Patch class extended for annotation of 
    a group of triangles in a Tetmesh. The area is the total area of the 
    encapsulated triangles.
    """    
    def __init__(self, *args, **kwargs): 
        """
        Construction::
        
            tmpatch = steps.geom.Comp(id, container, tris, icomp, ocomp = None)
            
        Construct a TmPatch object with identifier string id and assign container 
        as the parent geometry container. Assign icomp as the “inner” compartment 
        and assign also ocomp as the “outer” compartment (if required). 
            
        Arguments: 
            * string id
            * steps.geom.Tetmesh container
            * list<uint> tris
            * steps.geom.TmComp icomp
            * steps.geom.TmComp ocomp (default = None)
        """
        this = _steps_swig.new_TmPatch(*args, **kwargs)
        try: self.this.append(this)
        except: self.this = this
        self.thisown = False
        self.__swig_getmethods__["tris"] = _steps_swig.TmPatch_getAllTriIndices
        
    tris = steps_swig._swig_property(_steps_swig.TmPatch_getAllTriIndices)
    """List of indices of triangles associated to the patch."""
    
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

class DiffBoundary(steps_swig.DiffBoundary):
    """
    Annotation of a group of triangles in a Tetmesh. The triangles form 
    a boundary between two compartments, that may allow diffusion of some
    specified species.
    """    
    def __init__(self, *args, **kwargs): 
        """
        Construction::
        
            diffb = steps.geom.DiffBoundary(id, container, tris)
            
        Construct a DiffBoundary object with identifier string id and assign container 
        as the parent geometry container, described by group of triangles tris.
            
        Arguments: 
            * string id
            * steps.geom.Tetmesh container
            * list<uint> tris
        """
        this = _steps_swig.new_DiffBoundary(*args, **kwargs)
        try: self.this.append(this)
        except: self.this = this
        self.thisown = False
        self.__swig_getmethods__["tris"] = _steps_swig.DiffBoundary_getAllTriIndices
        self.__swig_setmethods__["id"] = _steps_swig.DiffBoundary_setID
        self.__swig_getmethods__["id"] = _steps_swig.DiffBoundary_getID
        self.__swig_getmethods__["container"] = _steps_swig.DiffBoundary_getContainer
        
    tris = steps_swig._swig_property(_steps_swig.DiffBoundary_getAllTriIndices)
    """List of indices of triangles associated to the patch."""
    id = steps_swig._swig_property(_steps_swig.DiffBoundary_getID, _steps_swig.DiffBoundary_setID)
    """Identifier string of the diffusion boundary."""
    container = steps_swig._swig_property(_steps_swig.DiffBoundary_getContainer)
    """Reference to parent steps.geom.Tetmesh container."""
    
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

"""
loadASCII.__doc__ = 
Reads a tetrahedral mesh from a simple ASCII format. Please refer to 
the documentation of steps.geom.saveASCII for more information on this
file format.  

PARAMETERS
    pathname
        The name of the file and its location.

RETURNS
    A steps.geom.Tetmesh object.

EXCEPTIONS
    steps.ArgErr
        Something is wrong with the pathname.
    steps.IOErr
        Cannot open the file or the file is badly formatted.

SEE ALSO
    steps.geom.saveASCII


saveASCII.__doc__ = 
saveASCII() saves a tetmesh to a very simple STEPS-centered ASCII 
(i.e. text based) format. The format is straightforward and consists 
of five parts, following the five parts of the tetmesh itself.

1/ A list of vertices: first the number of vertices is given. Then
   for each vertex there is a line with its x, y and z coordinates
   in scientific notation.

2/ A list of triangles. First the total number of triangles is given.  
   After this, we have 1 line per triangle giving its three corner 
   points as indices into the vertex list.

3/ A list of tetrahedrons. On a first line, the number of tets is 
   specified. After this, for each tet, follows a line with its four
   corner points, given as indices into the vertex list.

4/ The compartments in the mesh. First the number of compartments is
   given on a single line. Then each of these compartments is 
   specified using the following structure:
   
    a. Its name (ID string) on a single line.
    b. The number of volume systems that have been added to the
       compartment.
    c. For each volume system, its name (ID string) is given on a 
       separate line.
    d. The number of tetrahedrons associated with the compartment.
    e. The indices of these tetrahedrons (8 per line).

5/ A similar approach is taken to list all patches in the meshes. 
   first, we specify the number of patches attached to the mesh.
   Then, for each patch, we describe like this:
 
    a. The patch's name on a single line.
    b. If the patch has NO inner compartment, we print a "0"
       on a single line. Otherwise the line will start with a "1",
       followed by the name (ID string) of this inner compartment.
    c. The same for the outer compartment.
    d. The number of surface systems that have been added to the
       patch.
    e. For each of these surface systems, we specify its name on
       a separate line.
    f. The number of triangles in the patch.
    g. The indices of these triangles (8 per line).

All indices start from zero in this format, as they do in 
STEPS (and C++) internally.

PARAMETERS
    pathname
        The file and optionally the location that the mesh 
        will be written to.
    m
        A reference to a steps.geom.Tetmesh object.

RETURNS
    /

EXCEPTIONS
    steps.ArgErr
        The pathname is malformed or no mesh was specified.
    steps.IOErr
        Error while writing the mesh file (disk full; unable to 
        open file).

SEE ALSO
    steps.geom.loadASCII
"""

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

# END
